<?php
// This class was compiled from php.net comments and modified in April 2006 by
// Dominik Schulz <ds@gauner.org>
#  Description: Simple class using php mail function to construct and send mime multipart
#                emails (i.e. emails with attachments) and support content-id style
#                embedded images in html messages
#
#  Limitations: Uses the ubiquitously supported 7bit (i.e. no encoding) message encoding where as
#                qouted-printable is recommended for html messages. Does not ensure that message
#                line lengths do not exceed the 998 character limit specified by RFC 2822.
#
#  Usage Example:
#    $mulmail = new multipartmail("krisd@work.net", "destination@anywhere.com", "Some Subject");
#    $cid = $mulmail->addattachment("/var/www/html/img/pic.jpg", "image/jpg");
#    $mulmail->addmessage(
#      "<html>\n" .
#      "  <head>\n" .
#      "  </head>\n" .
#      "  <body>\n" .
#      "  This is text before<img src=\"cid:$cid\"> and after\n" .
#      "  </body>\n" .
#      "</html>\n", "text/html");
#    $mulmail->sendmail();

   class multipartmail{
     var $header;
     var $parts;
     var $message;
     var $subject;
     var $to_address;
     var $boundary;
     
     function qp_encoding($Message) {
     /* Build (most polpular) Extended ASCII Char/Hex MAP (characters >127 & <255) */
     for ($i=0; $i<127; $i++) {
	$CharList[$i] = "/".chr($i+128)."/";
	$HexList[$i] = "=".strtoupper(bin2hex(chr($i+128)));
     }
     /* Encode equal sign & 8-bit characters as equal signs followed by their hexadecimal values */
     $Message = str_replace("=", "=3D", $Message);
     $Message = preg_replace($CharList, $HexList, $Message);
     /* Lines longer than 76 characters (size limit for quoted-printable Content-Transfer-Encoding)
       will be cut after character 75 and an equals sign is appended to these lines. */
     $MessageLines = split("\n", $Message);
     $Message_qp = "";
     while(list(, $Line) = each($MessageLines)) {
       if (strlen($Line) > 75) {
           $Pointer = 0;       
           while ($Pointer <= strlen($Line)) {
               $Offset = 0;
               if (preg_match("/^=(3D|([8-9A-F]{1}[0-9A-F]{1}))$/", substr($Line, ($Pointer+73), 3))) $Offset=-2;
               if (preg_match("/^=(3D|([8-9A-F]{1}[0-9A-F]{1}))$/", substr($Line, ($Pointer+74), 3))) $Offset=-1;
               $Message_qp.= substr($Line, $Pointer, (75+$Offset))."=\n";
               if ((strlen($Line) - ($Pointer+75)) <= 75) {               
                   $Message_qp.= substr($Line, ($Pointer+75+$Offset))."\n";
                   break 1;
               }
               $Pointer+= 75+$Offset;
           }
         } else {
           $Message_qp.= $Line."\n";
         }
       }       
     return $Message_qp;
     }

     function multipartmail($dest, $src, $sub){
         $this->to_address = $dest;
         $this->subject = $sub;
         $this->parts = array("");
         $this->boundary = "------------" . md5(uniqid(time()));
         $this->header = "From: $src\r\n" .
                         "MIME-Version: 1.0\r\n" .
                         "Content-Type: multipart/related;\n" .
                         " boundary=\"" . $this->boundary . "\"\r\n" .
                         "X-Mailer: Multipartmail with PHP/" . phpversion();
     }
     
     function addbcc($dest) {
	     $this->header .= "\r\nBCC: ".$dest;
     }

     //function addmessage($msg = "", $ctype = "text/plain"){
    //     $this->parts[0] = "Content-Type: $ctype; charset=ISO-8859-1\r\n" .
    //                       "Content-Transfer-Encoding: 7bit\r\n" .
    //                       "\n" . $msg;
    //                       //chunk_split($msg, 68, "\n");  // disable this
    // }
    function addmessage($msg = "", $ctype = "text/plain") {
	$msg = $this->qp_encoding($msg);
         $this->parts[0] = "Content-Type: $ctype; charset=ISO-8859-1\r\n" .
                           "Content-Transfer-Encoding: quoted-printable\r\n" .
                          "\n" . $msg;
                          //chunk_split($msg, 68, "\n");  // disable this
    }

     function addattachment($file, $ctype){
         $fname = substr(strrchr($file, "/"), 1);
         $data = file_get_contents($file);
         $i = count($this->parts);
         $content_id = "part$i." . sprintf("%09d", crc32($fname)) . strrchr($this->to_address, "@");
         $this->parts[$i] = "Content-Type: $ctype; name=\"$fname\"\r\n" .
                           "Content-Transfer-Encoding: base64\r\n" .
                           "Content-ID: <$content_id>\r\n" .
                           "Content-Disposition: inline;\n" .
                           " filename=\"$fname\"\r\n" .
                           "\n" .
                           chunk_split( base64_encode($data), 68, "\n");
         return $content_id;
     }

     function buildmessage(){
         $this->message = "This is a multipart message in mime format.\n";
         $cnt = count($this->parts);
         for($i=0; $i<$cnt; $i++){
           $this->message .= "--" . $this->boundary . "\n" .
                             $this->parts[$i];
         }
     }

     /* to get the message body as a string */
     function getmessage(){
         $this->buildmessage();
         return $this->message;
     }

     function sendmail(){
         $this->buildmessage();
	 if(mail($this->to_address, $this->subject, $this->message, $this->header)) {
	  return true;
	} else {
	  return false;
	}
     }
   } // end-of-class "multipartmail"

// falls die Funktion zum bestimmten des MIME-Typs nicht existiert definieren wir sie uns hier
// Wir zum korrekten versenden von Mails mit Anhngen bentigt
if (!function_exists('mime_content_type'))
{
   function mime_content_type ( $f )
   {
       return system(trim('file -bi ' . escapeshellarg($f))) ;
   }
} 
?>